#include "ural.h"
#include "beama.h"

u_short portbase = 0; /*port base для не root серверов*/

int main (int argc, char *argv[]) {
  stderr = freopen(LOGFILE, "a", stderr); /*Перенаправляем поток ошибок в файл протокол*/
  char * service = PORT;
  struct sockaddr_in fsin; /*адрес клиента*/
  unsigned int alen; /*длина адреса клиента*/
  msock = ssock = 0;
  switch (argc) {
    case 1:
      break;
    case 2:
      service = argv[1];
      break;
    default:
      errexit("main: argc\n");
  }
  msock = passivesock(service, TRANSPORT, QLEN);
  (void) signal(SIGCHLD, reaper);
  while (1) {
    alen = sizeof(fsin);
    ssock = accept(msock, (struct sockaddr *)&fsin, &alen);
    if (ssock < 0) {
      if (errno == EINTR) continue;
      errexit("accept: error\n");
    }
    switch (fork()) {
      case 0: /*дочерний процесс*/
        (void) close(msock);
        exit(beama(ssock));
      default: /*родительский процесс*/
        (void) close(ssock);
        break;
      case -1: errexit("fork: error\n");
    }
  }
}

int passivesock(const char *service, const char *transport, int qlen) {
  struct servent * pse; /*указатель на сервисную информацию сервера*/
  struct protoent * ppe; /*указатель на информацию о протоколе*/
  struct sockaddr_in sin; /*адрес точки назначения*/
  int s, type; /*дескриптор сокета и тип сокета*/
  memset(&sin, 0, sizeof(sin));
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  if ((pse = getservbyname(service, transport)) != 0)
      sin.sin_port = htons(ntohs((u_short)pse->s_port) + portbase);
  else if ( (sin.sin_port = htons((u_short)atoi(service))) == 0 )
      errexit("getservbyname: can't get service entry\n");
  if ( (ppe = getprotobyname(transport)) == 0)
      errexit("getprotobyname: can't get protocol entry\n");
  if (strcmp(transport, TRANSPORT_UDP) == 0) type = SOCK_DGRAM;
  else type = SOCK_STREAM;
  s = socket(PF_INET, type, ppe->p_proto);
  if (s < 0) errexit("socket: can't create socket\n");
  if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0)
      errexit("bind: can't bind\n");
  if (type == SOCK_STREAM && listen(s, qlen) < 0)
      errexit("listen: can't listen\n");
  return s;
}

void reaper(int sig) {
  int status;
  while (wait3(&status, WNOHANG, (struct rusage *)0) >= 0);
}

int errexit(const char * errstr) {
  fprintf(stderr,"%s", errstr); fflush(stderr);
  if (msock > 0) (void) close(msock);
  if (ssock > 0) (void) close(ssock);
  exit(1);
}
